# -*- coding: utf-8 -*-
from plivo.utils import to_param_dict
from plivo.utils.validators import *

from ..base import ListResponseObject, PlivoResource, PlivoResourceInterface


class Endpoint(PlivoResource):
    _name = 'Endpoint'
    _identifier_string = 'endpoint_id'

    @validate_args(
        password=[of_type(six.text_type)],
        alias=[of_type(six.text_type)],
        app_id=[optional(of_type(six.text_type))])
    def update(self, password=None, alias=None, app_id=None):
        params = to_param_dict(self.update, locals())
        self.__dict__.update(params)
        return self.client.endpoints.update(self.id, **params)

    def delete(self):
        return self.client.endpoints.delete(self.id)


class Endpoints(PlivoResourceInterface):
    _resource_type = Endpoint

    @validate_args(
        username=[of_type(six.text_type)],
        password=[of_type(six.text_type)],
        alias=[of_type(six.text_type)],
        app_id=[optional(of_type(six.text_type))])
    def create(self, username, password, alias, app_id=None):
        return self.client.request('POST', ('Endpoint', ),
                                   to_param_dict(self.create, locals()))

    @validate_args(endpoint_id=[of_type(six.text_type)])
    def get(self, endpoint_id):
        return self.client.request('GET', ('Endpoint', endpoint_id))

    @validate_args(
        limit=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda limit: 0 < limit <= 20, '0 < limit <= 20')))
                ],
        offset=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda offset: 0 <= offset, '0 <= offset')))
        ])
    def list(self, limit=20, offset=0):
        return self.client.request(
            'GET',
            ('Endpoint', ),
            to_param_dict(self.list, locals()),
            objects_type=Endpoint,
            response_type=ListResponseObject)

    @validate_args(
        endpoint_id=[of_type(six.text_type)],
        password=[optional(of_type(six.text_type))],
        alias=[optional(of_type(six.text_type))],
        app_id=[optional(of_type(six.text_type))])
    def update(self, endpoint_id, password=None, alias=None, app_id=None):
        return self.client.request('POST', ('Endpoint', endpoint_id),
                                   to_param_dict(self.update, locals()))

    @validate_args(endpoint_id=[of_type(six.text_type)])
    def delete(self, endpoint_id):
        return self.client.request('DELETE', ('Endpoint', endpoint_id))



from plivo.xml import (
    PlivoXMLElement,
    map_type,
    SpeakElement,
    PlayElement,
    WaitElement
)
from plivo.utils.validators import *


class GetDigitsElement(PlivoXMLElement):
    _name = 'GetDigits'
    _nestable = [
        'Speak',
        'Play',
        'Wait'
    ]

    @property
    def action(self):
        return self.__action

    @action.setter
    def action(self, value):
        self.__action = six.text_type(value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_action(self, value):
        self.action = value
        return self

    @property
    def method(self):
        return self.__method

    @method.setter
    def method(self, value):
        self.__method = six.text_type(value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_method(self, value):
        self.method = value
        return self

    @property
    def timeout(self):
        return self.__timeout

    @timeout.setter
    def timeout(self, value):
        self.__timeout = int(value) if value is not None else None

    @validate_args(
        value=[of_type(*six.integer_types)],
    )
    def set_timeout(self, value):
        self.timeout = value
        return self

    @property
    def digit_timeout(self):
        return self.__digit_timeout

    @digit_timeout.setter
    def digit_timeout(self, value):
        self.__digit_timeout = int(value) if value is not None else None

    @validate_args(
        value=[of_type(*six.integer_types)],
    )
    def set_digit_timeout(self, value):
        self.digit_timeout = value
        return self

    @property
    def finish_on_key(self):
        return self.__finish_on_key

    @finish_on_key.setter
    def finish_on_key(self, value):
        self.__finish_on_key = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_finish_on_key(self, value):
        self.finish_on_key = value
        return self

    @property
    def num_digits(self):
        return self.__num_digits

    @num_digits.setter
    def num_digits(self, value):
        self.__num_digits = int(value) if value is not None else None

    @validate_args(
        value=[of_type(*six.integer_types)],
    )
    def set_num_digits(self, value):
        self.num_digits = value
        return self

    @property
    def retries(self):
        return self.__retries

    @retries.setter
    def retries(self, value):
        self.__retries = int(value) if value is not None else None

    @validate_args(
        value=[of_type(*six.integer_types)],
    )
    def set_retries(self, value):
        self.retries = value
        return self

    @property
    def redirect(self):
        return self.__redirect

    @redirect.setter
    def redirect(self, value):
        self.__redirect = bool(value) if value is not None else None

    @validate_args(
        value=[of_type_exact(bool)],
    )
    def set_redirect(self, value):
        self.redirect = value
        return self

    @property
    def play_beep(self):
        return self.__play_beep

    @play_beep.setter
    def play_beep(self, value):
        self.__play_beep = bool(value) if value is not None else None

    @validate_args(
        value=[of_type_exact(bool)],
    )
    def set_play_beep(self, value):
        self.play_beep = value
        return self

    @property
    def valid_digits(self):
        return self.__valid_digits

    @valid_digits.setter
    def valid_digits(self, value):
        self.__valid_digits = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_valid_digits(self, value):
        self.valid_digits = value
        return self

    @property
    def invalid_digits_sound(self):
        return self.__invalid_digits_sound

    @invalid_digits_sound.setter
    def invalid_digits_sound(self, value):
        self.__invalid_digits_sound = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_invalid_digits_sound(self, value):
        self.invalid_digits_sound = value
        return self

    @property
    def log(self):
        return self.__log

    @log.setter
    def log(self, value):
        self.__log = bool(value) if value is not None else None

    @validate_args(
        value=[of_type_exact(bool)],
    )
    def set_log(self, value):
        self.log = value
        return self

    def __init__(
            self,
            action=None,
            method=None,
            timeout=None,
            digit_timeout=None,
            finish_on_key=None,
            num_digits=None,
            retries=None,
            redirect=None,
            play_beep=None,
            valid_digits=None,
            invalid_digits_sound=None,
            log=None,
    ):
        super(GetDigitsElement, self).__init__()

        self.action = action
        self.method = method
        self.timeout = timeout
        self.digit_timeout = digit_timeout
        self.finish_on_key = finish_on_key
        self.num_digits = num_digits
        self.retries = retries
        self.redirect = redirect
        self.play_beep = play_beep
        self.valid_digits = valid_digits
        self.invalid_digits_sound = invalid_digits_sound
        self.log = log

    def to_dict(self):
        d = {
            'action': self.action,
            'method': self.method,
            'timeout': self.timeout,
            'digitTimeout': self.digit_timeout,
            'finishOnKey': self.finish_on_key,
            'numDigits': self.num_digits,
            'retries': self.retries,
            'redirect': self.redirect,
            'playBeep': self.play_beep,
            'validDigits': self.valid_digits,
            'invalidDigitsSound': self.invalid_digits_sound,
            'log': self.log,
        }
        return {
            k: six.text_type(map_type(v))
            for k, v in d.items() if v is not None
        }

    def add_speak(
            self,
            content,
            voice=None,
            language=None,
            loop=None,
    ):
        self.add(
            SpeakElement(
                content=content,
                voice=voice,
                language=language,
                loop=loop,
            ))
        return self

    def add_play(
            self,
            content,
            loop=None,
    ):
        self.add(PlayElement(
            content=content,
            loop=loop,
        ))
        return self

    def add_wait(
            self,
            length=None,
            silence=None,
            min_silence=None,
            beep=None,
    ):

        self.add(
            WaitElement(
                length=length,
                silence=silence,
                min_silence=min_silence,
                beep=beep,
            ))
        return self



from plivo.xml import PlivoXMLElement, map_type
from plivo.utils.validators import *


class MessageElement(PlivoXMLElement):
    _name = 'Message'
    _nestable = []

    @property
    def src(self):
        return self.__src

    @src.setter
    def src(self, value):
        self.__src = six.text_type(value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_src(self, value):
        self.src = value
        return self

    @property
    def dst(self):
        return self.__dst

    @dst.setter
    def dst(self, value):
        self.__dst = six.text_type(value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_dst(self, value):
        self.dst = value
        return self

    @property
    def type(self):
        return self.__type

    @type.setter
    def type(self, value):
        self.__type = six.text_type(value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_type(self, value):
        self.type = value
        return self

    @property
    def callback_url(self):
        return self.__callback_url

    @callback_url.setter
    def callback_url(self, value):
        self.__callback_url = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_callback_url(self, value):
        self.callback_url = value
        return self

    @property
    def callback_method(self):
        return self.__callback_method

    @callback_method.setter
    def callback_method(self, value):
        self.__callback_method = six.text_type(
            value) if value is not None else None

    @validate_args(
        value=[of_type(six.text_type)],
    )
    def set_callback_method(self, value):
        self.callback_method = value
        return self

    def __init__(
            self,
            content,
            src=None,
            dst=None,
            type=None,
            callback_url=None,
            callback_method=None,
    ):
        super(MessageElement, self).__init__()

        self.content = content
        self.src = src
        self.dst = dst
        self.type = type
        self.callback_url = callback_url
        self.callback_method = callback_method

    def to_dict(self):
        d = {
            'src': self.src,
            'dst': self.dst,
            'type': self.type,
            'callbackUrl': self.callback_url,
            'callbackMethod': self.callback_method,
        }
        return {
            k: six.text_type(map_type(v))
            for k, v in d.items() if v is not None
        }



# -*- coding: utf-8 -*-
"""
Base client, used for all API requests.
"""

import os
import platform
from collections import namedtuple

from plivo.base import ResponseObject
from plivo.exceptions import (AuthenticationError, InvalidRequestError,
                              PlivoRestError, PlivoServerError,
                              ResourceNotFoundError, ValidationError)
from plivo.utils import is_valid_mainaccount, is_valid_subaccount
from plivo.version import __version__
from requests import Request, Session

AuthenticationCredentials = namedtuple('AuthenticationCredentials',
                                       'auth_id auth_token')

PLIVO_API = 'https://api.plivo.com'
PLIVO_API_BASE_URI = '/'.join([PLIVO_API, 'v1/Account'])


def get_user_agent():
    return 'plivo-python/%s (Python: %s)' % (__version__,
                                             platform.python_version())


def fetch_credentials(auth_id, auth_token):
    """Fetches the right credentials either from params or from environment"""

    if not (auth_id and auth_token):
        try:
            auth_id = os.environ['PLIVO_AUTH_ID']
            auth_token = os.environ['PLIVO_AUTH_TOKEN']
        except KeyError:
            raise AuthenticationError('The Plivo Python SDK '
                                      'could not find your auth credentials.')

    if not (is_valid_mainaccount(auth_id) or is_valid_subaccount(auth_id)):
        raise AuthenticationError('Invalid auth_id supplied: %s' % auth_id)

    return AuthenticationCredentials(auth_id=auth_id, auth_token=auth_token)


class BaseClient(object):
    def __init__(self, auth_id=None, auth_token=None, proxies=None, timeout=5):
        """
        The Plivo API client.

        Deals with all the API requests to be made.
        """

        self.base_uri = PLIVO_API_BASE_URI
        self.session = Session()
        self.session.headers.update({
            'User-Agent': get_user_agent(),
            'Content-Type': 'application/json',
            'Accept': 'application/json',
        })
        self.session.auth = fetch_credentials(auth_id, auth_token)
        self.multipart_session = Session()
        self.multipart_session.headers.update({
            'User-Agent': get_user_agent(),
            'Cache-Control': 'no-cache',
            'Content-Type': 'multipart/form-data',
        })
        self.multipart_session.auth = fetch_credentials(auth_id, auth_token)
        self.proxies = proxies
        self.timeout = timeout

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.session.close()
        self.multipart_session.close()

    def process_response(self,
                         method,
                         response,
                         response_type=None,
                         objects_type=None):
        """Processes the API response based on the status codes and method used
        to access the API
        """

        try:
            response_json = response.json(
                object_hook=lambda x: ResponseObject(x) if isinstance(x, dict) else x)
            if response_type:
                r = response_type(self, response_json.__dict__)
                response_json = r

            if 'objects' in response_json and objects_type:
                response_json.objects = [
                    objects_type(self, obj.__dict__)
                    for obj in response_json.objects
                ]
        except ValueError:
            response_json = None

        if response.status_code == 400:
            if response_json and 'error' in response_json:
                raise ValidationError(response_json.error)
            raise ValidationError(
                'A parameter is missing or is invalid while accessing resource'
                'at: {url}'.format(url=response.url))

        if response.status_code == 401:
            if response_json and 'error' in response_json:
                raise AuthenticationError(response_json.error)
            raise AuthenticationError(
                'Failed to authenticate while accessing resource at: '
                '{url}'.format(url=response.url))

        if response.status_code == 404:
            if response_json and 'error' in response_json:
                raise ResourceNotFoundError(response_json.error)
            raise ResourceNotFoundError(
                'Resource not found at: {url}'.format(url=response.url))

        if response.status_code == 405:
            if response_json and 'error' in response_json:
                raise InvalidRequestError(response_json.error)
            raise InvalidRequestError(
                'HTTP method "{method}" not allowed to access resource at: '
                '{url}'.format(method=method, url=response.url))

        if response.status_code == 500:
            if response_json and 'error' in response_json:
                raise PlivoServerError(response_json.error)
            raise PlivoServerError(
                'A server error occurred while accessing resource at: '
                '{url}'.format(url=response.url))

        if method == 'DELETE':
            if response.status_code != 204:
                raise PlivoRestError('Resource at {url} could not be '
                                     'deleted'.format(url=response.url))

        elif response.status_code not in [200, 201, 202]:
            raise PlivoRestError(
                'Received status code {status_code} for the HTTP method '
                '"{method}"'.format(
                    status_code=response.status_code, method=method))

        return response_json

    def create_request(self, method, path=None, data=None):
        path = path or []
        req = Request(method, '/'.join([self.base_uri, self.session.auth[0]] +
                                       list([str(p) for p in path])) + '/',
                      **({
                          'params': data
                      } if method == 'GET' else {
                          'json': data
                      }))
        return self.session.prepare_request(req)

    def create_multipart_request(self,
                                 method,
                                 path=None,
                                 data=None,
                                 files=None):
        path = path or []

        data_args = {}
        if method == 'GET':
            data_args['params'] = data
        else:
            data_args['data'] = data
            if files and 'file' in files and files['file'] != '':
                data_args['files'] = files

        req = Request(method,
                      '/'.join([self.base_uri, self.multipart_session.auth[0]]
                               + list([str(p) for p in path])) + '/', **(
                                   data_args))
        return self.multipart_session.prepare_request(req)

    def send_request(self, request, **kwargs):
        if 'session' in kwargs:
            session = kwargs['session']
            del kwargs['session']
        else:
            session = self.session

        return session.send(
            request, proxies=self.proxies, timeout=self.timeout, **kwargs)

    def request(self,
                method,
                path=None,
                data=None,
                response_type=None,
                objects_type=None,
                files=None,
                **kwargs):
        if files is not None:
            req = self.create_multipart_request(method, path, data, files)
            session = self.multipart_session
        else:
            req = self.create_request(method, path, data)
            session = self.session
        kwargs['session'] = session
        res = self.send_request(req, **kwargs)
        return self.process_response(method, res, response_type, objects_type)



# -*- coding: utf-8 -*-
from plivo.utils import to_param_dict
from plivo.utils.validators import *

from ..base import ListResponseObject, PlivoResource, PlivoResourceInterface


class Address(PlivoResource):
    _name = 'Address'
    _identifier_string = 'id'

    def update(self,
               salutation=None,
               first_name=None,
               last_name=None,
               country_iso=None,
               address_line1=None,
               address_line2=None,
               city=None,
               region=None,
               postal_code=None,
               alias=None,
               file_to_upload=None,
               auto_correct_address=None,
               callback_url=None):
        return self.client.addresses.update(
            self.id, salutation, first_name, last_name, country_iso,
            address_line1, address_line2, city, region, postal_code, alias,
            file_to_upload, auto_correct_address, callback_url)

    def delete(self):
        return self.client.addresses.delete(self.id)


class Addresses(PlivoResourceInterface):
    _resource_type = Address

    @validate_args(
        country_iso=[of_type(six.text_type)],
        salutation=[all_of(of_type(six.text_type), is_in(('Mr', 'Ms')))],
        first_name=[of_type(six.text_type)],
        last_name=[of_type(six.text_type)],
        address_line1=[of_type(six.text_type)],
        address_line2=[of_type(six.text_type)],
        city=[of_type(six.text_type)],
        region=[of_type(six.text_type)],
        postal_code=[of_type(six.text_type)],
        address_proof_type=[
            all_of(
                of_type(six.text_type),
                is_in(('national_id', 'passport', 'business_id', 'NIF', 'NIE',
                       'DNI', 'others')))
        ],
        alias=[optional(of_type(six.text_type))],
        file_to_upload=[optional(of_type(six.text_type))],
        auto_correct_address=[optional(of_type_exact(bool))],
        fiscal_identification_code=[optional(of_type(six.text_type))],
        street_code=[optional(of_type(six.text_type))],
        municipal_code=[optional(of_type(six.text_type))])
    def create(self,
               country_iso,
               salutation,
               first_name,
               last_name,
               address_line1,
               address_line2,
               city,
               region,
               postal_code,
               address_proof_type,
               alias=None,
               file_to_upload=None,
               auto_correct_address=None,
               fiscal_identification_code=None,
               street_code=None,
               municipal_code=None,
               callback_url=None):
        if country_iso == 'ES' and not fiscal_identification_code:
            raise ValidationError(
                'The parameter fiscal_identification_code is required for Spain numbers'
            )

        if country_iso == 'DK' and not all(street_code, municipal_code):
            raise ValidationError(
                'The parameters street_code and municipal_code are required for Denmark numbers'
            )

        if file_to_upload:
            file_extension = file_to_upload.strip().split('.')[-1].lower()
            if file_extension not in ['jpg', 'jpeg', 'png', 'pdf']:
                raise ValidationError(
                    'File format of the file to be uploaded should be one of JPG, JPEG, PNG or PDF'
                )

            content_types = {
                'jpeg': 'image/jpeg',
                'jpg': 'image/jpeg',
                'png': 'image/png',
                'pdf': 'application/pdf',
            }

            import os

            files = {
                'file': (file_to_upload.split(os.sep)[-1], open(
                    file_to_upload, 'rb'), content_types[file_extension])
            }
        else:
            files = {'file': ''}
        data_to_send = to_param_dict(self.create, locals())
        return self.client.request(
            'POST', ('Verification', 'Address'), data_to_send, files=files)

    @validate_args(
        address_id=[of_type(six.text_type)],
        salutation=[all_of(of_type(six.text_type), is_in(('Mr', 'Ms')))],
        first_name=[of_type(six.text_type)],
        last_name=[of_type(six.text_type)],
        country_iso=[optional(of_type(six.text_type))],
        address_line1=[optional(of_type(six.text_type))],
        address_line2=[optional(of_type(six.text_type))],
        city=[optional(of_type(six.text_type))],
        region=[optional(of_type(six.text_type))],
        postal_code=[optional(of_type(six.text_type))],
        alias=[optional(of_type(six.text_type))],
        file_to_upload=[optional(of_type(six.text_type))],
        auto_correct_address=[optional(of_type_exact(bool))], )
    def update(self,
               address_id,
               salutation=None,
               first_name=None,
               last_name=None,
               country_iso=None,
               address_line1=None,
               address_line2=None,
               city=None,
               region=None,
               postal_code=None,
               alias=None,
               file_to_upload=None,
               auto_correct_address=None,
               callback_url=None):
        if file_to_upload:
            file_extension = file_to_upload.strip().split('.')[-1].lower()
            if file_extension not in ['jpg', 'jpeg', 'png', 'pdf']:
                raise ValidationError(
                    'File format of the file to be uploaded should be one of JPG, JPEG, PNG or PDF'
                )

            content_types = {
                'jpeg': 'image/jpeg',
                'jpg': 'image/jpeg',
                'png': 'image/png',
                'pdf': 'application/pdf',
            }

            import os

            files = {
                'file': (file_to_upload.split(os.sep)[-1], open(
                    file_to_upload, 'rb'), content_types[file_extension])
            }
        else:
            files = {'file': ''}

        data_to_send = to_param_dict(self.create, locals())
        return self.client.request(
            'POST', ('Verification', 'Address', address_id),
            data_to_send,
            files=files)

    @validate_args(address_id=[of_type(six.text_type)])
    def get(self, address_id):
        return self.client.request(
            'GET', ('Verification', 'Address', address_id),
            response_type=Address)

    @validate_args(
        country_iso=[optional(of_type(six.text_type))],
        customer_name=[optional(of_type(six.text_type))],
        alias=[optional(of_type(six.text_type))],
        verification_status=[
            optional(
                all_of(
                    of_type(six.text_type),
                    is_in(('pending', 'accepted', 'rejected', ))))
        ],
        validation_status=[
            optional(
                all_of(
                    of_type(six.text_type),
                    is_in(('pending', 'accepted', 'rejected', ))))
        ],
        limit=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda limit: 0 < limit <= 20, '0 < limit <= 20')))
        ],
        offset=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda offset: 0 <= offset, '0 <= offset')))
        ])
    def list(self,
             country_iso=None,
             customer_name=None,
             alias=None,
             verification_status=None,
             validation_status=None,
             limit=20,
             offset=0):
        return self.client.request(
            'GET', ('Verification', 'Address', ),
            to_param_dict(self.list, locals()),
            response_type=ListResponseObject,
            objects_type=Address)

    @validate_args(address_id=[of_type(six.text_type)])
    def delete(self, address_id):
        return self.client.request('DELETE', ('Verification', 'Address',
                                              address_id))



from setuptools import find_packages, setup

long_description = '''\
The Plivo Python SDK makes it simpler to integrate communications into your
Python applications using the Plivo REST API. Using the SDK, you will be able
to make voice calls, send SMS and generate Plivo XML to control your call flows.

See https://github.com/plivo/plivo-python for more information.
'''

setup(
    name='plivo',
    version='4.8.0',
    description='A Python SDK to make voice calls & send SMS using Plivo and to generate Plivo XML',
    long_description=long_description,
    url='https://github.com/plivo/plivo-python',
    author='The Plivo SDKs Team',
    author_email='sdks@plivo.com',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Telecommunications Industry',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Communications :: Telephony',
    ],
    install_requires=[
        'requests >= 2, < 3',
        'six >= 1, < 2',
        'decorator >= 4, < 5',
        'lxml >= 3, < 5',
    ],
    keywords=['plivo', 'plivo xml', 'voice calls', 'sms'],
    include_package_data=True,
    packages=find_packages(exclude=['tests', 'tests.*']), )



from plivo.base import PlivoResource, PlivoResourceInterface

from ..utils.validators import *


class QueuedCall(PlivoResource):
    _name = 'QueuedCall'
    _identifier_string = 'call_uuid'


class QueuedCalls(PlivoResourceInterface):
    _resource_type = QueuedCall
    _iterable = False

    @validate_args(
        limit=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(
                        lambda limit: 0 < limit <= 20,
                        message='0 < limit <= 20')))
        ],
        offset=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda offset: 0 <= offset, message='0 <= offset')))
        ])
    def list_ids(self, limit=20, offset=0):
        return self.client.request('GET', ('Call', ), {
            'status': 'queued',
            'limit': limit,
            'offset': offset,
        })

    @validate_args(_id=[of_type(six.text_type)])
    def get(self, _id):
        return self.client.request('GET', ('Call', _id), {'status': 'queued'})



# -*- coding: utf-8 -*-
from plivo.utils.validators import *

from ..base import ListResponseObject, PlivoResource, PlivoResourceInterface, ResponseObject
from ..exceptions import *
from ..utils import *
from plivo.resources.numberpools import NumberPool

class Powerpack(PlivoResource):
    _name = 'Powerpack'
    _identifier_string = 'uuid'
    numberpool = None

    def get(self):
        return self.client.powerpacks.get(self.id)

    def delete(self, unrent_numbers=False):
        params ={}
        params['unrent_numbers'] = unrent_numbers
        return self.client.request('DELETE', ('Powerpack', self.id),
                                    params,  response_type=None,objects_type=None)
    # params values should be dictionary like {'sticky_sender': false, 'name':"hello"} 
    def update(self,params=None):
        if params == None:
            raise ValidationError(
                'required atleast one of powerpack attributes'
            )
        return self.client.request('POST', ('Powerpack', self.id),
                                   params, response_type=None,
                objects_type=None)

    def get_numberpooluuid(self):
        powerpack = self.client.request(
            'GET', ('Powerpack', self.id), response_type=Powerpack)
        numberpoolpath = powerpack.number_pool
        numberpool_uuid = ""
        if numberpoolpath:
            numberpool = numberpoolpath.split('/')
            numberpool_uuid = numberpool[5]
            return numberpool_uuid
        return ""

    @validate_args(
        limit=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda limit: 0 < limit <= 20, '0 < limit <= 20')))
        ],
        offset=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda offset: 0 <= offset, '0 <= offset')))
        ])
        

    def list_numbers(self,
        starts_with=None, 
        country_iso2=None,
        type=None, limit=None, offset=None):
        numberpool_uuid = self.get_numberpooluuid()
        params ={}
        if starts_with:
            params['starts_with'] = starts_with
        if country_iso2:
            params['country_iso2'] = country_iso2
        if type:
            params['type'] = type
        if limit:
            params['limit']= limit
        if offset:
            params['offset'] = offset
        if numberpool_uuid != "":
            return self.client.request('GET', ('NumberPool',numberpool_uuid,'Number'),
                params ,response_type=None,
                objects_type=None)
        else:
            raise ResourceNotFoundError('Resource not found')

    def count_numbers(self,
        starts_with=None, 
        country_iso2=None,
        type=None):
        numberpool_uuid = self.get_numberpooluuid()
        params ={}
        if starts_with:
            params['starts_with'] = starts_with
        if country_iso2:
            params['country_iso2'] = country_iso2
        if type:
            params['type'] = type
        if numberpool_uuid != "":
            try:
                response = self.client.request(
                'GET', ('NumberPool',numberpool_uuid,'Number'),
                params ,response_type=None,
                objects_type=None)
                return response['meta']['total_count']
            except:
                return 0
        else:
            raise ResourceNotFoundError('Resource not found')


    @validate_args(
    number=[of_type(six.text_type)]
    )
    def find_number(self, number):
        numberpool_uuid = self.get_numberpooluuid()
        if numberpool_uuid != "":
            return self.client.request(
            'GET', ('NumberPool',numberpool_uuid,'Number', number),
            response_type=None,
            objects_type=None)
        else:
            raise ResourceNotFoundError('Resource not found')

    @validate_args(
    number=[of_type(six.text_type)]
    )
    def add_number(self,  number):
        numberpool_uuid = self.get_numberpooluuid()
        if numberpool_uuid != "":
            return self.client.request(
            'POST', ('NumberPool',numberpool_uuid,'Number', number),
            response_type=None,
            objects_type=Powerpack)
        else:
            raise ResourceNotFoundError('Resource not found')

    @validate_args(
    tollfree=[of_type(six.text_type)]
    )
    def add_tollfree(self,  tollfree):
        numberpool_uuid = self.get_numberpooluuid()
        if numberpool_uuid != "":
            return self.client.request(
            'POST', ('NumberPool',numberpool_uuid,'Tollfree', tollfree),
            response_type=None,
            objects_type=None)
        else:
            raise ResourceNotFoundError('Resource not found')

    @validate_args(
    number=[of_type(six.text_type)]
    )
    def remove_number(self,  number, unrent=False):
        numberpool_uuid = self.get_numberpooluuid()
        params = {}
        params['unrent'] = unrent
        if numberpool_uuid != "":
            return self.client.request(
            'DELETE', ('NumberPool',numberpool_uuid,'Number', number), params,
            response_type=None,
            objects_type=None)
        else:
            raise ResourceNotFoundError('Resource not found')

    @validate_args(
    shortcode=[of_type(six.text_type)]
    )
    def remove_shortcode(self,  shortcode):
        numberpool_uuid = self.get_numberpooluuid()
        if numberpool_uuid != "":
            return self.client.request(
            'DELETE', ('NumberPool',numberpool_uuid,'Shortcode', shortcode), 
            response_type=None,
            objects_type=None)
        else:
            raise ResourceNotFoundError('Resource not found')

    @validate_args(
    tollfree=[of_type(six.text_type)]
    )
    def remove_tollfree(self,  tollfree, unrent=False):
        numberpool_uuid = self.get_numberpooluuid()
        params = {}
        params['unrent'] = unrent
        if numberpool_uuid != "":
            return self.client.request(
            'DELETE', ('NumberPool',numberpool_uuid,'Tollfree', tollfree), params,
            response_type=None,
            objects_type=None)
        else:
            raise ResourceNotFoundError('Resource not found')


    @validate_args(
        limit=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda limit: 0 < limit <= 20, '0 < limit <= 20')))
        ],
        offset=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda offset: 0 <= offset, '0 <= offset')))
        ])
    def list_shortcodes(self,  limit=20, offset=0):
        params ={}
        params['limit'] = limit
        params['offset'] = offset
        numberpool_uuid = self.get_numberpooluuid()
        if numberpool_uuid != "":
            return self.client.request(
            'GET', ('NumberPool',numberpool_uuid,'Shortcode'),
            params,
            response_type=None,
            objects_type=None)
        else:
            raise ResourceNotFoundError('Resource not found')

    @validate_args(
        limit=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda limit: 0 < limit <= 20, '0 < limit <= 20')))
        ],
        offset=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda offset: 0 <= offset, '0 <= offset')))
        ])
    def list_tollfree(self,  limit=20, offset=0):
        params ={}
        params['limit'] = limit
        params['offset'] = offset
        numberpool_uuid = self.get_numberpooluuid()
        if numberpool_uuid != "":
            return self.client.request(
            'GET', ('NumberPool',numberpool_uuid,'Tollfree'),
            params,
            response_type=None,
            objects_type=None)
        else:
            raise ResourceNotFoundError('Resource not found')

    @validate_args(
        shortcode=[of_type(six.text_type)])
    def find_shortcode(self,  shortcode):
        numberpool_uuid = self.get_numberpooluuid()
        if numberpool_uuid != "":
            return self.client.request(
            'GET', ('NumberPool',numberpool_uuid,'Shortcode', shortcode),
            response_type=None,
            objects_type=None)
        else:
            raise ResourceNotFoundError('Resource not found')
    
    @validate_args(
        tollfree=[of_type(six.text_type)])
    def find_tollfree(self,  tollfree):
        numberpool_uuid = self.get_numberpooluuid()
        if numberpool_uuid != "":
            return self.client.request(
            'GET', ('NumberPool',numberpool_uuid,'Tollfree', tollfree),
            response_type=None,
            objects_type=None)
        else:
            raise ResourceNotFoundError('Resource not found')



    def buy_add_number(self,
        pattern=None,
        country_iso2=None,
        type=None, 
        region=None, 
        number=''):
        numberpool_uuid = self.get_numberpooluuid()
        params = {}
        params['rent'] = 'true'
        if numberpool_uuid != "":   
            if number !="":
                return self.client.request(
                'POST', ('NumberPool',numberpool_uuid,'Number', number),params,
                response_type=None,
                objects_type=None)
            else :
                phonenumberparam = {}
                if pattern:
                    phonenumberparam['pattern'] = pattern
                if type:
                    phonenumberparam['type'] = type
                if country_iso2:
                    phonenumberparam['country_iso'] = country_iso2
                if region:
                    phonenumberparam['region'] = region
                number_response = self.client.request(
                        'GET',
                        ('PhoneNumber', ),
                        phonenumberparam,
                        objects_type=None,
                        response_type=None, )
                if len(number_response)< 1:
                    raise  ResourceNotFoundError('Resource not found')
                number = number_response[0]['number']
                return self.client.request(
                'POST', ('NumberPool',numberpool_uuid,'Number', number),params,
                response_type=None,
                objects_type=None)
        else:
            raise ResourceNotFoundError('Resource not found')
     
    
class Powerpacks(PlivoResourceInterface):
    _resource_type = Powerpack
    @validate_args(
        sticky_sender=[optional(of_type_exact(bool))],
        local_connect=[optional(of_type_exact(bool))],
        application_type=[optional(of_type(six.text_type))],
        application_id=[optional(of_type(six.text_type))])
    def create(self,
               name,
               sticky_sender=True,
               local_connect=True,
               application_type='',
               application_id=''):
        if (name is None):
            raise ValidationError(
                'name parameter is invalid'
            )
        return self.client.request('POST', ('Powerpack', ),
                                   to_param_dict(self.create, locals()))
    
    @validate_args(uuid=[of_type(six.text_type)])
    def get(self, uuid):
        response = self.client.request(
            'GET', ('Powerpack', uuid), response_type=Powerpack)
        numberpoolpath = response.number_pool
        number_pool_id = ""
        if numberpoolpath:
            numberpool = numberpoolpath.split('/')
            number_pool_id = numberpool[5]
            data = {
            'number_pool_id': number_pool_id
        }
            Powerpack.numberpool = NumberPool( self.client, data)
        return response

    @validate_args(
        limit=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda limit: 0 < limit <= 20, '0 < limit <= 20')))
        ],
        offset=[
            optional(
                all_of(
                    of_type(*six.integer_types),
                    check(lambda offset: 0 <= offset, '0 <= offset')))
        ])
    def list(self,
             limit=None,
             offset=None):
        return self.client.request(
            'GET', ('Powerpack', ),
            to_param_dict(self.list, locals()),
            response_type=None,
            objects_type=None)
 
    
